@@ -2,13 +2,12 @@ import os
 
 from pytorch_lightning.utilities import _module_available
 
-EXAMPLES_ROOT = os.path.dirname(__file__)
-PACKAGE_ROOT = os.path.dirname(EXAMPLES_ROOT)
-DATASETS_PATH = os.path.join(PACKAGE_ROOT, 'Datasets')
-
-TORCHVISION_AVAILABLE = _module_available("torchvision")
-DALI_AVAILABLE = _module_available("nvidia.dali")
+_EXAMPLES_ROOT = os.path.dirname(__file__)
+_PACKAGE_ROOT = os.path.dirname(_EXAMPLES_ROOT)
+_DATASETS_PATH = os.path.join(_PACKAGE_ROOT, 'Datasets')
 
+_TORCHVISION_AVAILABLE = _module_available("torchvision")
+_DALI_AVAILABLE = _module_available("nvidia.dali")
 
 LIGHTNING_LOGO = """
                     ####
