@@ -13,17 +13,18 @@
 # limitations under the License.
 import platform
 from typing import Optional
+from warnings import warn
 
 from torch.utils.data import DataLoader, random_split
 
-from pl_examples import DATASETS_PATH, TORCHVISION_AVAILABLE
+from pl_examples import _DATASETS_PATH, _TORCHVISION_AVAILABLE
 from pytorch_lightning import LightningDataModule
 
-if TORCHVISION_AVAILABLE:
+if _TORCHVISION_AVAILABLE:
     from torchvision import transforms as transform_lib
     from torchvision.datasets import MNIST
 else:
-    from tests.base.datasets import MNIST
+    from tests.helpers.datasets import MNIST
 
 
 class MNISTDataModule(LightningDataModule):
@@ -38,7 +39,7 @@ class MNISTDataModule(LightningDataModule):
 
     def __init__(
         self,
-        data_dir: str = DATASETS_PATH,
+        data_dir: str = _DATASETS_PATH,
         val_split: int = 5000,
         num_workers: int = 16,
         normalize: bool = False,
@@ -55,8 +56,12 @@ class MNISTDataModule(LightningDataModule):
             normalize: If true applies image normalize
         """
         super().__init__(*args, **kwargs)
-        if platform.system() == "Windows":
-            # see: https://stackoverflow.com/a/59680818/4521646
+        if num_workers and platform.system() == "Windows":
+            # see: https://stackoverflow.com/a/59680818
+            warn(
+                f"You have requested num_workers={num_workers} on Windows,"
+                " but currently recommended is 0, so we set it for you"
+            )
             num_workers = 0
 
         self.dims = (1, 28, 28)
@@ -126,12 +131,12 @@ class MNISTDataModule(LightningDataModule):
 
     @property
     def default_transforms(self):
-        if not TORCHVISION_AVAILABLE:
+        if not _TORCHVISION_AVAILABLE:
             return None
         if self.normalize:
-            mnist_transforms = transform_lib.Compose(
-                [transform_lib.ToTensor(), transform_lib.Normalize(mean=(0.5,), std=(0.5,))]
-            )
+            mnist_transforms = transform_lib.Compose([
+                transform_lib.ToTensor(), transform_lib.Normalize(mean=(0.5, ), std=(0.5, ))
+            ])
         else:
             mnist_transforms = transform_lib.ToTensor()
 
