@@ -11,7 +11,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """
 Model Checkpointing
 ===================
@@ -19,8 +18,7 @@ Model Checkpointing
 Automatically save model checkpoints during training.
 
 """
-import logging
-import numbers
+
 import os
 import re
 from copy import deepcopy
@@ -31,13 +29,14 @@ import numpy as np
 import torch
 import yaml
 
+from pytorch_lightning import _logger as log
 from pytorch_lightning.callbacks.base import Callback
-from pytorch_lightning.metrics.metric import Metric
 from pytorch_lightning.utilities import rank_zero_info, rank_zero_only, rank_zero_warn
 from pytorch_lightning.utilities.cloud_io import get_filesystem
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
+from pytorch_lightning.utilities.warnings import WarningCache
 
-log = logging.getLogger(__name__)
+warning_cache = WarningCache()
 
 
 class ModelCheckpoint(Callback):
@@ -48,46 +47,6 @@ class ModelCheckpoint(Callback):
     best checkpoint file and :attr:`best_model_score` to retrieve its score.
 
     Args:
-        filepath: path to save the model file.
-
-            .. warning:: .. deprecated:: 1.0
-
-               Use ``dirpath`` + ``filename`` instead. Will be removed in v1.2
-
-        monitor: quantity to monitor. By default it is ``None`` which saves a checkpoint only for the last epoch.
-        verbose: verbosity mode. Default: ``False``.
-        save_last: When ``True``, always saves the model at the end of the epoch to
-            a file `last.ckpt`. Default: ``None``.
-        save_top_k: if ``save_top_k == k``,
-            the best k models according to
-            the quantity monitored will be saved.
-            if ``save_top_k == 0``, no models are saved.
-            if ``save_top_k == -1``, all models are saved.
-            Please note that the monitors are checked every `period` epochs.
-            if ``save_top_k >= 2`` and the callback is called multiple
-            times inside an epoch, the name of the saved file will be
-            appended with a version count starting with `v0`.
-        mode: one of {auto, min, max}.
-            If ``save_top_k != 0``, the decision
-            to overwrite the current save file is made
-            based on either the maximization or the
-            minimization of the monitored quantity. For `val_acc`,
-            this should be `max`, for `val_loss` this should
-            be `min`, etc. In `auto` mode, the direction is
-            automatically inferred from the name of the monitored quantity.
-
-            .. warning::
-               Setting ``mode='auto'`` has been deprecated in v1.1 and will be removed in v1.3.
-
-        save_weights_only: if ``True``, then only the model's weights will be
-            saved (``model.save_weights(filepath)``), else the full model
-            is saved (``model.save(filepath)``).
-        period: Interval (number of epochs) between checkpoints.
-        prefix: A string to put at the beginning of checkpoint filename.
-
-            .. warning::
-               This argument has been deprecated in v1.1 and will be removed in v1.3
-
         dirpath: directory to save the model file.
 
             Example::
@@ -114,7 +73,58 @@ class ModelCheckpoint(Callback):
                 ... )
 
             By default, filename is ``None`` and will be set to ``'{epoch}-{step}'``.
+        monitor: quantity to monitor. By default it is ``None`` which saves a checkpoint only for the last epoch.
+        verbose: verbosity mode. Default: ``False``.
+        save_last: When ``True``, always saves the model at the end of the epoch to
+            a file `last.ckpt`. Default: ``None``.
+        save_top_k: if ``save_top_k == k``,
+            the best k models according to
+            the quantity monitored will be saved.
+            if ``save_top_k == 0``, no models are saved.
+            if ``save_top_k == -1``, all models are saved.
+            Please note that the monitors are checked every ``period`` epochs.
+            if ``save_top_k >= 2`` and the callback is called multiple
+            times inside an epoch, the name of the saved file will be
+            appended with a version count starting with ``v1``.
+        mode: one of {auto, min, max}.
+            If ``save_top_k != 0``, the decision
+            to overwrite the current save file is made
+            based on either the maximization or the
+            minimization of the monitored quantity. For `val_acc`,
+            this should be `max`, for `val_loss` this should
+            be `min`, etc. In `auto` mode, the direction is
+            automatically inferred from the name of the monitored quantity.
 
+            .. warning::
+               Setting ``mode='auto'`` has been deprecated in v1.1 and will be removed in v1.3.
+
+        save_weights_only: if ``True``, then only the model's weights will be
+            saved (``model.save_weights(filepath)``), else the full model
+            is saved (``model.save(filepath)``).
+        period: Interval (number of epochs) between checkpoints.
+        prefix: A string to put at the beginning of checkpoint filename.
+
+            .. warning::
+               This argument has been deprecated in v1.1 and will be removed in v1.3
+
+    Note:
+        For extra customization, ModelCheckpoint includes the following attributes:
+
+        - ``CHECKPOINT_JOIN_CHAR = "-"``
+        - ``CHECKPOINT_NAME_LAST = "last"``
+        - ``FILE_EXTENSION = ".ckpt"``
+        - ``STARTING_VERSION = 1``
+
+        For example, you can change the default last checkpoint name by doing
+        ``checkpoint_callback.CHECKPOINT_NAME_LAST = "{epoch}-last"``
+
+    Raises:
+        MisconfigurationException:
+            If ``save_top_k`` is neither ``None`` nor more than or equal to ``-1``,
+            if ``monitor`` is ``None`` and ``save_top_k`` is none of ``None``, ``-1``, and ``0``, or
+            if ``mode`` is none of ``"min"``, ``"max"``, and ``"auto"``.
+        ValueError:
+            If ``trainer.save_checkpoint`` is ``None``.
 
     Example::
 
@@ -139,15 +149,18 @@ class ModelCheckpoint(Callback):
         model = ...
         trainer.fit(model)
         checkpoint_callback.best_model_path
+
     """
 
     CHECKPOINT_JOIN_CHAR = "-"
     CHECKPOINT_NAME_LAST = "last"
     FILE_EXTENSION = ".ckpt"
+    STARTING_VERSION = 1
 
     def __init__(
         self,
-        filepath: Optional[str] = None,
+        dirpath: Optional[Union[str, Path]] = None,
+        filename: Optional[str] = None,
         monitor: Optional[str] = None,
         verbose: bool = False,
         save_last: Optional[bool] = None,
@@ -156,8 +169,6 @@ class ModelCheckpoint(Callback):
         mode: str = "auto",
         period: int = 1,
         prefix: str = "",
-        dirpath: Optional[Union[str, Path]] = None,
-        filename: Optional[str] = None,
     ):
         super().__init__()
         self.monitor = monitor
@@ -177,9 +188,6 @@ class ModelCheckpoint(Callback):
         self.save_function = None
         self.warned_result_obj = False
 
-        if save_top_k is None and monitor is not None:
-            self.save_top_k = 1
-
         if prefix:
             rank_zero_warn(
                 'Argument `prefix` is deprecated in v1.1 and will be removed in v1.3.'
@@ -187,7 +195,7 @@ class ModelCheckpoint(Callback):
             )
 
         self.__init_monitor_mode(monitor, mode)
-        self.__init_ckpt_dir(filepath, dirpath, filename, save_top_k)
+        self.__init_ckpt_dir(dirpath, filename, save_top_k)
         self.__validate_init_configuration()
 
     def on_pretrain_routine_start(self, trainer, pl_module):
@@ -255,9 +263,7 @@ class ModelCheckpoint(Callback):
 
     def __validate_init_configuration(self):
         if self.save_top_k is not None and self.save_top_k < -1:
-            raise MisconfigurationException(
-                f'Invalid value for save_top_k={self.save_top_k}. Must be None or >= -1'
-            )
+            raise MisconfigurationException(f'Invalid value for save_top_k={self.save_top_k}. Must be None or >= -1')
         if self.monitor is None:
             # None: save last epoch, -1: save all epochs, 0: nothing is saved
             if self.save_top_k not in [None, -1, 0]:
@@ -271,41 +277,15 @@ class ModelCheckpoint(Callback):
                     ' You can save the last checkpoint with ModelCheckpoint(save_top_k=None, monitor=None).'
                 )
 
-    def __init_ckpt_dir(self, filepath, dirpath, filename, save_top_k):
-        if filepath:
-            if (dirpath or filename):
-                raise MisconfigurationException(
-                    'You have set all three path/name inputs which are not feasible.'
-                    f' You have to choose either filepath={filepath} OR dirpath={dirpath}'
-                    f' and filename={filename} configuration.'
-                )
-
-            rank_zero_warn(
-                'Argument `filepath` is deprecated in v1.0 and will be removed in v1.2.'
-                ' Please use `dirpath` and `filename` instead.', DeprecationWarning
-            )
-
-            _fs = get_filesystem(filepath)
-
-            if _fs.isdir(filepath):
-                dirpath, filename = filepath, None
-            else:
-                if _fs.protocol == 'file':
-                    filepath = os.path.realpath(filepath)
-                dirpath, filename = os.path.split(filepath)
+    def __init_ckpt_dir(self, dirpath, filename, save_top_k):
 
         self._fs = get_filesystem(str(dirpath) if dirpath else '')
 
         if (
-            save_top_k is not None
-            and save_top_k > 0
-            and dirpath is not None
-            and self._fs.isdir(dirpath)
+            save_top_k is not None and save_top_k > 0 and dirpath is not None and self._fs.isdir(dirpath)
             and len(self._fs.ls(dirpath)) > 0
         ):
-            rank_zero_warn(
-                f"Checkpoint directory {dirpath} exists and is not empty."
-            )
+            rank_zero_warn(f"Checkpoint directory {dirpath} exists and is not empty.")
 
         if dirpath and self._fs.protocol == 'file':
             dirpath = os.path.realpath(dirpath)
@@ -320,26 +300,18 @@ class ModelCheckpoint(Callback):
             "max": (-torch_inf, "max"),
         }
 
-        # TODO: Update with MisconfigurationException when auto mode is removed in v1.3
         if mode not in mode_dict and mode != 'auto':
-            rank_zero_warn(
-                f"ModelCheckpoint mode {mode} is unknown, fallback to auto mode",
-                RuntimeWarning,
-            )
-            mode = "auto"
+            raise MisconfigurationException(f"`mode` can be auto, {', '.join(mode_dict.keys())}, got {mode}")
 
+        # TODO: Update with MisconfigurationException when auto mode is removed in v1.3
         if mode == 'auto':
             rank_zero_warn(
                 "mode='auto' is deprecated in v1.1 and will be removed in v1.3."
-                " Default value for mode with be 'min' in v1.3.",
-                DeprecationWarning
+                " Default value for mode with be 'min' in v1.3.", DeprecationWarning
             )
 
-            mode_dict['auto'] = (
-                (-torch_inf, "max")
-                if monitor is not None and ("acc" in monitor or monitor.startswith("fmeasure"))
-                else (torch_inf, "min")
-            )
+            _condition = monitor is not None and ("acc" in monitor or monitor.startswith("fmeasure"))
+            mode_dict['auto'] = ((-torch_inf, "max") if _condition else (torch_inf, "min"))
 
         self.kth_value, self.mode = mode_dict[mode]
 
@@ -350,6 +322,7 @@ class ModelCheckpoint(Callback):
             log.debug(f"Removed checkpoint: {filepath}")
 
     def _save_model(self, filepath: str, trainer, pl_module):
+        # Todo: required argument `pl_module` is not used
         # in debugging, track when we save checkpoints
         trainer.dev_debugger.track_checkpointing_history(filepath)
 
@@ -414,9 +387,7 @@ class ModelCheckpoint(Callback):
 
         return filename
 
-    def format_checkpoint_name(
-        self, epoch: int, step: int, metrics: Dict[str, Any], ver: Optional[int] = None
-    ) -> str:
+    def format_checkpoint_name(self, epoch: int, step: int, metrics: Dict[str, Any], ver: Optional[int] = None) -> str:
         """Generate a filename according to the defined template.
 
         Example::
@@ -439,9 +410,7 @@ class ModelCheckpoint(Callback):
             'step=0.ckpt'
 
         """
-        filename = self._format_checkpoint_name(
-            self.filename, epoch, step, metrics, prefix=self.prefix
-        )
+        filename = self._format_checkpoint_name(self.filename, epoch, step, metrics, prefix=self.prefix)
         if ver is not None:
             filename = self.CHECKPOINT_JOIN_CHAR.join((filename, f"v{ver}"))
 
@@ -462,6 +431,7 @@ class ModelCheckpoint(Callback):
         The base path gets extended with logger name and version (if these are available)
         and subfolder "checkpoints".
         """
+        # Todo: required argument `pl_module` is not used
         if self.dirpath is not None:
             return  # short circuit
 
@@ -474,15 +444,12 @@ class ModelCheckpoint(Callback):
 
             version = (
                 trainer.logger.version
-                if isinstance(trainer.logger.version, str)
-                else f"version_{trainer.logger.version}"
+                if isinstance(trainer.logger.version, str) else f"version_{trainer.logger.version}"
             )
 
-            version, name = trainer.accelerator_backend.broadcast((version, trainer.logger.name))
+            version, name = trainer.training_type_plugin.broadcast((version, trainer.logger.name))
 
-            ckpt_path = os.path.join(
-                save_dir, str(name), version, "checkpoints"
-            )
+            ckpt_path = os.path.join(save_dir, str(name), version, "checkpoints")
         else:
             ckpt_path = os.path.join(trainer.weights_save_path, "checkpoints")
 
@@ -493,17 +460,23 @@ class ModelCheckpoint(Callback):
 
     def _add_backward_monitor_support(self, trainer):
         metrics = trainer.logger_connector.callback_metrics
+        deprecation_warning = False
 
-        # backward compatibility... need to deprecate
         if self.monitor is None and 'val_loss' in metrics:
             self.monitor = 'val_loss'
-
-        if self.monitor is None and 'checkpoint_on' in metrics:
-            self.monitor = 'checkpoint_on'
+            deprecation_warning = True
 
         if self.save_top_k is None and self.monitor is not None:
+            # TODO: Remove `Optional` from `save_top_k` when this is deleted in v1.4
             self.save_top_k = 1
 
+        if deprecation_warning:
+            warning_cache.warn(
+                "Relying on `self.log('val_loss', ...)` to set the ModelCheckpoint monitor is deprecated in v1.2"
+                " and will be removed in v1.4. Please, create your own `mc = ModelCheckpoint(monitor='your_monitor')`"
+                " and use it as `Trainer(callbacks=[mc])`.", DeprecationWarning
+            )
+
     def _validate_monitor_key(self, trainer):
         metrics = trainer.logger_connector.callback_metrics
 
@@ -526,7 +499,7 @@ class ModelCheckpoint(Callback):
     ) -> str:
         filepath = self.format_checkpoint_name(epoch, step, ckpt_name_metrics)
 
-        version_cnt = 0
+        version_cnt = self.STARTING_VERSION
         while self.file_exists(filepath, trainer) and filepath != del_filepath:
             filepath = self.format_checkpoint_name(epoch, step, ckpt_name_metrics, ver=version_cnt)
             version_cnt += 1
@@ -534,11 +507,9 @@ class ModelCheckpoint(Callback):
         return filepath
 
     def _monitor_candidates(self, trainer):
-        ckpt_name_metrics = deepcopy(trainer.logger_connector.logged_metrics)
-        ckpt_name_metrics.update(trainer.logger_connector.callback_metrics)
-        ckpt_name_metrics.update(trainer.logger_connector.progress_bar_metrics)
-        ckpt_name_metrics.update({"step": trainer.global_step, "epoch": trainer.current_epoch})
-        return ckpt_name_metrics
+        monitor_candidates = deepcopy(trainer.logger_connector.callback_metrics)
+        monitor_candidates.update(step=trainer.global_step, epoch=trainer.current_epoch)
+        return monitor_candidates
 
     def _save_last_checkpoint(self, trainer, pl_module, ckpt_name_metrics):
         should_save_last = self.monitor is None or self.save_last
@@ -557,21 +528,20 @@ class ModelCheckpoint(Callback):
             last_filepath = os.path.join(self.dirpath, f"{last_filepath}{self.FILE_EXTENSION}")
         else:
             last_filepath = self._get_metric_interpolated_filepath_name(
-                ckpt_name_metrics, trainer.current_epoch, trainer.global_step, trainer,
+                ckpt_name_metrics,
+                trainer.current_epoch,
+                trainer.global_step,
+                trainer,
             )
 
-        accelerator_backend = trainer.accelerator_backend
-
-        if accelerator_backend is not None and accelerator_backend.rpc_enabled:
+        if trainer.training_type_plugin.rpc_enabled:
             # RPCPlugin manages saving all model states
-            accelerator_backend.ddp_plugin.rpc_save_model(self._save_model, last_filepath, trainer, pl_module)
+            trainer.training_type_plugin.rpc_save_model(self._save_model, last_filepath, trainer, pl_module)
         else:
             self._save_model(last_filepath, trainer, pl_module)
         if (
-                self.last_model_path
-                and self.last_model_path != last_filepath
-                and (self.save_top_k != -1 or self.save_last)
-                and trainer.is_global_zero
+            self.last_model_path and self.last_model_path != last_filepath
+            and (self.save_top_k != -1 or self.save_last) and trainer.is_global_zero
         ):
             self._del_model(self.last_model_path)
         self.last_model_path = last_filepath
@@ -584,30 +554,24 @@ class ModelCheckpoint(Callback):
         epoch = metrics.get("epoch")
         step = metrics.get("step")
 
-        if current is not None:
-            if isinstance(current, Metric):
-                current = current.compute()
-            elif isinstance(current, numbers.Number):
-                current = torch.tensor(current, device=pl_module.device, dtype=torch.float)
+        # when `val_loss` is being logged and no ModelCheckpoint is being provided
+        # `val_loss` will be selected for monitor and need to be reduced to
+        # prevent processes divergence
+        # TODO: Move this logic to logger_connector. This also needs to be fixed for any
+        # other monitor logged value which aren't produced from a Metric.
+        if self.monitor == "val_loss":
+            current = trainer.training_type_plugin.reduce(current, reduce_op="mean")
 
         if self.check_monitor_top_k(current):
             self._update_best_and_save(current, epoch, step, trainer, pl_module, metrics)
         elif self.verbose:
-            rank_zero_info(
-                f"Epoch {epoch:d}, step {step:d}: {self.monitor} was not in top {self.save_top_k}"
-            )
+            rank_zero_info(f"Epoch {epoch:d}, step {step:d}: {self.monitor} was not in top {self.save_top_k}")
 
     def _is_valid_monitor_key(self, metrics):
         return self.monitor in metrics or len(metrics) == 0
 
     def _update_best_and_save(
-        self,
-        current: torch.Tensor,
-        epoch: int,
-        step: int,
-        trainer,
-        pl_module,
-        ckpt_name_metrics
+        self, current: torch.Tensor, epoch: int, step: int, trainer, pl_module, ckpt_name_metrics
     ):
         k = len(self.best_k_models) + 1 if self.save_top_k == -1 else self.save_top_k
 
@@ -617,7 +581,7 @@ class ModelCheckpoint(Callback):
             self.best_k_models.pop(del_filepath)
 
         # do not save nan, replace with +/- inf
-        if torch.isnan(current):
+        if isinstance(current, torch.Tensor) and torch.isnan(current):
             current = torch.tensor(float('inf' if self.mode == "min" else '-inf'))
 
         filepath = self._get_metric_interpolated_filepath_name(ckpt_name_metrics, epoch, step, trainer, del_filepath)
@@ -629,9 +593,7 @@ class ModelCheckpoint(Callback):
         if len(self.best_k_models) == k:
             # monitor dict has reached k elements
             _op = max if self.mode == "min" else min
-            self.kth_best_model_path = _op(
-                self.best_k_models, key=self.best_k_models.get
-            )
+            self.kth_best_model_path = _op(self.best_k_models, key=self.best_k_models.get)
             self.kth_value = self.best_k_models[self.kth_best_model_path]
 
         _op = min if self.mode == "min" else max
@@ -665,6 +627,5 @@ class ModelCheckpoint(Callback):
         the internal state to diverge between ranks.
         """
         exists = self._fs.exists(filepath)
-        if trainer.accelerator_backend is not None:
-            exists = trainer.accelerator_backend.broadcast(exists)
+        exists = trainer.training_type_plugin.broadcast(exists)
         return exists
