@@ -13,7 +13,8 @@
 # limitations under the License.
 import torch
 
-from pytorch_lightning.metrics.utils import reduce as __reduce, class_reduce as __cr
+from pytorch_lightning.metrics.utils import class_reduce as __cr
+from pytorch_lightning.metrics.utils import reduce as __reduce
 from pytorch_lightning.utilities import rank_zero_warn
 
 
